;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@Keyboard=02BB			;KEYBOARD SCANNING subroutine
@Decode=07BD			;KEYBOARD DECODE subroutine
@Laddr=09D8			;LINE ADDRESS subroutine
@NextOne=09F2			;Find next variable routine
@Scanning=0F55			;The expression evaluator
@FPtoA=15CD			;Floating-point to A subroutine
@Errsp=4002			;Address of 1st item on machine stack
@DFile=400C			;Address of display file
@VARS=4010			;Address of VARS
@CHAdd=4016			;Address of next chr to be interpreted

@Max
	00			;Item count
@Width
	00			;Item character width
@Pik
	00			;Item Selected
@Chr
	00			;CLS character
;================================
; Jump Table
;================================
	JR Sortkeys		;16520, Sort out the keys pressed
	JP ListBox		;16522, Print listbox
	JP ListUp		;16525, Scroll list up
	JP ListDown		;16528, Scroll list down
	JP Marker		;16531, Print marker
	JP FillBox		;16534, Fill listbox with names
	JR CLS		        ;16537, CLS routine
	JR Pause		;16539, PAUSE routine
	JR Release		;16541, Key release routine
	JR SaveScr		;16543, Save screen to line 2
	JR LoadScr		;16545, Load screen from line 2
;================================
; CLS screen
;================================
@CLS
	LD A,(Chr)		;Get CLS character
	LD (CLSChr+1),A		;Set CLS character
	LD HL,(DFile)		;Get DFile
	LD BC,0319		;Byte counter
@CLSLoop
	BIT 6,(HL)		;N/L?
	JR NZ +2		;No, move on
@CLSChr
	LD (HL),00		;Clear DFile byte with (Chr) character
	INC HL			;Next DFile addr
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	RET Z			;Loop until BC=0
	JR CLSLoop
;================================
; Wait for key press
; On exit...
;  A=key chr code
;================================
@Pause
	CALL KeyScan		;Get keypress
	CP 01			;No key?
	JR Z Pause		;Yes, loop until a keypressed
	JR Ret0
;================================
; Wait for key release
;================================
@Release
	CALL Keyboard		;Call ROM scan keyboard routine.
	INC L			;Check if user's finger on key.
	JR NZ Release		;If so, go back and check again.
@Ret0
	LD BC,0000
	RET			;RET to BASIC, BC=0000
;================================
; Save screen
;================================
@SaveScr
	LD HL,0002		;Locate screen storage area
	CALL LAddr		;Use ROM routine to find line address
	LD BC,0006		;Skip header
	ADD HL,BC
	EX DE,HL		;DE=Destination
	LD HL,(DFile)		;HL=Source
@Save01
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	RET			;RET to BASIC, BC=0000
;================================
; Load screen
;================================
@LoadScr
	LD HL,0002		;Locate screen storage area
	CALL LAddr		;Use ROM routine to find line address
	LD BC,0006		;Skip header
	ADD HL,BC		;HL=screen store=source
	LD DE,(DFile)		;DE=DFile=Destination
	JR Save01
;================================
; Keyboard loop
;================================
@SortKeys
	CALL Pause		;Wait for keypress
	LD HL,KeyTable		;Table address
	LD BC,00120	;Table size
	CPIR
	JR NZ SortKeys
	LD C,(HL)		;Get key value
	RET			;Return key value to BASIC
;================================
@KeyTable			;Key followed by        GOSUB line
	"M" 14			;M-Pointer down		GOSUB 20
	"6" 14			;6-Pointer down
	"K" 1E			;K-Pointer up		GOSUB 30
	"7" 1E			;7-Pointer up
	"D" 28			;D-Page down		GOSUB 40
	"8" 28			;8-Page down
	"U" 32			;U-Page up		GOSUB 50
	"5" 32			;5-Page up
	 76 46			;N/L-Select		GOSUB 70
	 79 50			;Sh+N/L-Alt select	GOSUB 80
	"Q" 5A			;Q-Quit			GOSUB 100
	"0" 5A			;0-Quit
;Extra keys added for the Op-Art keyboard loop
	"I" 6E			;I-Input delay value	GOSUB 110
	"S" 78			;S-Slow/Fast mode	GOSUB 120
	"F" 7C			;F-Fast mode		GOSUB 124
	"P" 82			;P-Pallet switch	GOSUB 130
;================================
; Test for keypress.
;
; On exit...
;
;  A=key chr code
;
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Print the listbox
;================================
@ListBox
	LD A,(Width)		;Get width
	LD (WidthB+1),A		;Set width
	LD HL,(DFile)		;Set destination
	INC HL			;DFile+1
	LD DE,Header		;Listbox data
	CALL LineX		;Print line 0
	CALL LineX		;Print line 1
	CALL LineX		;Print line 2
	LD B,14			;Loop counter
@BodyTxt
	PUSH BC			;PUSH counter
	LD DE,Body		;Line 3 to 22 data
	CALL LineX		;Print lines 3 to 22
	POP BC			;POP counter
	DJNZ BodyTxt		;Loop Until B=0 then print line 23
@LineX
	PUSH HL			;PUSH DFile addr
	EX DE,HL		;DE=DFile HL=Table
	LD BC,0003		;Counter
	LDIR			;Print 1st three chrs
	EX DE,HL		;HL=DFile DE=Table
	LD A,(DE)		;Get chr to print
@WidthB
	LD B,00			;B=Width counter
	LD (HL),A		;Print chr
	INC HL			;Next DFile addr
	DJNZ WidthB+2		;Loop until B=0
	INC DE			;End chr addr
	LD A,(DE)		;Get end chr
	LD (HL),A		;Print end chr
	POP HL			;POP DFile
	LD BC,0021		;Offset to next row, 33 bytes
	ADD HL,BC		;Next row addr
	INC DE			;Next Table addr
	RET
;================================
; Fill ListBox with items
;================================
@FillBox
	LD A,(Max)
	LD B,A
	SUB 15
	JR C +2
	LD B,14
	LD A,B
	LD (FillLoop-1),A
	LD DE,(VARS)
@NextVar
	EX DE,HL
	LD A,(HL) 
	CP C7			;B$() found?
	JR Z +5			;Yes, move on
	CALL NextOne		;Find next item in VARS
	JR NextVar		;Loop until B$() found
	LD BC,0008		;Skip array header
	ADD HL,BC		;HL=Start of list
	LD (Item1+1),HL
	LD A,(Width)
	LD (FillC+1),A
	LD C,A			;BC=Item chr width
	LD A,(Pik)		;Get item number
@ListLoop
	DEC A			;Decrease item number
	JR Z PrintList		;If 0 move on
	ADD HL,BC		;Move to Pik item in list
	JR ListLoop
@PrintList
	EX DE,HL
	LD HL,(DFile)
	LD BC,0067		;Offset to NW corner of listbox
	ADD HL,BC		;NW corner of listbox
	EX DE,HL		;DE=Listbox addr, HL=Item addr
	LD A,00
@FillLoop
	PUSH AF			;PUSH row counter
	PUSH DE			;PUSH DFile addr
@FillC
	LD C,00			;BC=Chr width
	LD A,(HL)		;Test for EOL
	CP F3
	JR NZ +3
@Item1
	LD HL,0000		;Source=Item 1 address
	LDIR
	EX DE,HL		;DE=Item pointer
	POP HL			;HL=DFile addr
	LD C,21			;Bytes/row
	ADD HL,BC
	EX DE,HL		;DE=DFile, HL=Item pointer
	POP AF			;POP row counter
	DEC A			;Decrease row counter
	JR NZ FillLoop		;Loop until A=0
	LD BC,0000
	RET
;================================
; Listbox data
;================================
@Header
	87 83 83 83 83		;Line 0
	85 80 80 80 80		;Line 1
	85 B0 07 03 84		;Line 2
@Body
	85 08 05 00 85		;Line 3 to 22
	85 B2 82 83 81		;Line 23
;================================
; Scroll listbox up
;================================
@ListUp
	LD A,(Width)		;Get width
	INC A
	LD (WU+1),A		;Set width counter
	LD HL,(DFile)		;Get DFile
	LD DE,0066		;Offset to NW corner of listbox
	ADD HL,DE		;NW corner of listbox
	LD A,13			;Row counter, 19
@UpLoop
	LD BC,0021		;Bytes/ROW, 33
	LD D,H			;DE=destination
	LD E,L
	ADD HL,BC		;HL=source
	PUSH HL			;PUSH source
@WU
	LD C,00			;Characters/row
 	LDIR			;Move row up
	POP HL			;POP source
	DEC A			;Decrease row counter
	JR NZ UpLoop		;Loop until A=0
	RET			;RET 0 to BASIC
;================================
; Scroll listbox down
;================================
@ListDown
	LD A,(Width)		;Get width
	INC A
	LD (WD+1),A		;Set width counter
	LD D,00			;DE=Width
	LD E,A
	LD HL,02D8		;HL=22*33+2=728
	ADD HL,DE		;HL=HL+width
	EX DE,HL		;DE=Offset to SW corner of listbox
	LD HL,(DFile)		;Get DFile
	ADD HL,DE		;SW corner of listbox
	LD B,13			;ROW count, 19
@LDLoop
	LD A,B			;Save row counter in A
	LD BC,0021		;Bytes/ROW, 33
	LD D,H			;DE=destination
	LD E,L		
	AND A			;Clear carry
	SBC HL,BC		;HL=source addr.
	PUSH HL			;PUSH source
@WD
	LD C,00			;Characters/row
 	LDDR			;Move row up
	POP HL			;POP source
	LD B,A			;B=row counter
	DJNZ LDLoop		;Loop until B=0
	RET			;RET to BASIC, BC=0000
;================================
;Set Reg A to 8bit parameter value
;================================
@SetA
	LD (CHAdd),HL		;Start of parameter
	CALL Scanning		;Call the expression evaluator
	PUSH HL 		;PUSH BASIC line addr
	CALL FPtoA		;A=Results
	POP HL 			;POP BASIC line addr
	RET
;================================
; USAGE:
;
;  PRINT USR Marker;Y,CODE C$
;
;   Y=Ycoord
;   C$=Chr to POKE
;================================
@Marker
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;Ycoord addr
	CALL SetA		;Get Ycoord+3
	ADD A,03
	LD B,A			;B=row counter
	INC HL			;CHR$ addr
	PUSH HL			;PUSH CHadd
	LD HL,(DFile)		;Get DFile+1
	INC HL
	LD DE,0021		;33 bytes/row
@AddRow
	ADD HL,DE
	DJNZ AddRow
	INC HL			;Cursor position
@Rows0
	LD (PokeAddr+1),HL	;Set DFile Ycoord offset
	POP HL			;POP CHadd
	CALL SetA		;Get CHR$
@PokeAddr
	LD HL,0000
	LD (HL),A		;POKE DFile
@SPRet
	LD SP,(Errsp)		;Set stack pointer to ROMs "NEXT-LINE" routine
	RET			;Return via "NEXT-LINE" routine
	00