;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;===============================

ORG 48B5

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Find line address subroutine
@Print1=0AD5			;PRINT-1 subroutine
@Scanning=0F55			;The expression evaluator
@FPtoA=15CD			;Floating-point to A subroutine
@Errsp=4002			;Address of 1st item on machine stack
@DFile=400C			;Address of display file
@DFxCC=400E			;Address of PRINT position in display file
@Vars=4010			;Start of VARS
@CHAdd=4016			;Address of next chr to be interpreted
@ChroF=4021			;Chroma flag
@PRBuff=403C			;Attribute 0 store
;================================
@JumpTable
	JR ChromaMode		;18613
	JR ChromaTest		;18615
	JR Rite			;18617
	JR Left			;18619
	JR Down			;18621
	JR Up			;18623
	JP Box			;18625
	JP Ink			;18628
	JP Hues			;18631
	JP Invert		;18634
	JP KeySort		;18637
	JP PokeDF		;18640
	00 00 00
	00 00 00
;================================
;Chroma test
;================================
@ChromaMode
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;HL=X1 parameter
	CALL SetA		;Get starting Xcoord
	LD (Mode+1),A		;Set mode
        LD BC,7FEF		;Chroma color input port.
@Mode
	LD A,00 		;Set color mode
	OUT (c),A 
@SPRet
	LD SP,(Errsp)		;Set stack pointer to ROMs "NEXT-LINE" routine
	RET			;Return via "NEXT-LINE" routine
;================================
;Chroma test
;================================
@ChromaTest
        LD BC,7FEF		;Chroma color input port.
	IN A,(C) 
	BIT 5,A
	LD BC,0000
	RET NZ			;IF NOT Chroma RETURN 0000
	INC C
	RET			;IF Chroma RETURN 0001
;================================
;Rite, draw horizontal line left to right
;
; USAGE:
;
;  PRINT AT Y,X;USR Rite;W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   W=Width, 1 to 255,  0=Error B
;   C$=Chr to print
;================================
@Rite
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
@PrintR
	RST 10			;Print A (VARS overflow causes Error 5)
	DJNZ PrintR		;Loop until B=0
	JP LineEnd
;================================
;Left, draw horizontal line right to left
;
; USAGE:
;
;  PRINT AT Y,X;USR Left;W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   W=Width, 1 to 255,  0=Error B
;   C$=Chr to print
;================================
@Left
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
	RST 10			;Print A
	LD HL,(DFxCC)		;Get DFxCC
	DEC HL			;Move print position back by 2
	DEC HL
	BIT 6,(HL)		;EOL?
	JR Z +1			;No, move on
	DEC HL			;Backup one more
	LD (DFxCC),HL		;Set (DFxCC)=next print position
	EX DE,HL		;Test for DFile underflow
	LD HL,(DFile)		;Get DFile
	SBC HL,DE		;CHAdd<DFile?
	JR NC +2		;Yes, underflow condition, stop printing
	DJNZ Left+4		;Loop until B=0
	JP LineEnd		
;================================
;Down/Up, draw vertical lines
;
; USAGE:
;
;  PRINT AT Y,X;USR Down;H,CODE C$
;  PRINT AT Y,X;USR Up;H,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   H=Height, 1 to 24, 0=Error B
;   C$=Chr to print
;================================
@Down
	LD A,20			;Offset value
	LD HL,0019		;Add instruction
	JR +5
@Up
	LD A,22			;Offset value
	LD HL,52ED		;Sub instruction
	LD (SetV+1),A		;Set offset value
	LD (AddSub),HL		;Set Add/Sub instruction
	CALL GetPars		;Get line pars, A=CODE X$ B=Length
	PUSH HL			;PUSH CHAdd
@SetV
	LD DE,0000		;Offset to next line
	RST 10			;Print A
	LD HL,(DFxCC)		;Get DFxCC
	AND A			;Clear CY
@AddSub
	00 00			;Add or Subtract line offset
	LD (DFxCC),HL		;Set DFxCC
	EX DE,HL
	LD HL,(DFile)		;Upper DFile overflow
	SBC HL,DE
	JR NC LineEnd		;If overflow stop printing
	LD HL,(Vars)		;Lower DFile overflow
	SBC HL,DE
	JR C LineEnd		;If overflow stop printing
	DJNZ SetV		;Loop until B=0
	JR LineEnd			
;================================
;Box, draw solid rectangle
;
; USAGE:
;
;  PRINT AT Y,X;USR Box;H,W,CODE C$
;   Y=Ycoord
;   X=Xcoord
;   H=Height, 1 to 24, 0=Error B
;   W=Width
;   C$=Fill Chr
;================================
@Box
	CALL GetPars		;Get line pars, A=C=Width  B=height
	AND A			;Width=0?
	JP Z ErrorB		;Yes, error B
	LD E,(IY+57)		;Get SPosnX
	DEC E			;E=maximum allowable length
	SUB E			;Width>Max?
	JR C +1			;No, move on
	LD C,E			;Set width to max
	LD A,C
	LD (Width+1),A		;Set Width
	LD A,B			;A=Height
	LD (Width-1),A		;Set Height
	LD A,21			;33 bytes/row
	SUB C			;C=Width
	LD (Offset+1),A		;Set offset, offset=33-width
	INC HL			;Skip seperator
	CALL SetA		;Get fill chr
	PUSH HL			;PUSH CHAdd
	LD C,00			;C=Height
@Width
	LD B,00			;Loop counter
	RST 10			;Print A
	DJNZ Width+2		;Print A until B=0
	DEC C			;Decrease height counter
	JP Z LineEnd		;When C=0 check for EOL
	LD HL,(DFxCC)		;Get address of PRINT position
@Offset
	LD DE,0000
	ADD HL,DE		;Add offset
	EX DE,HL		;DE=DFxCC
	LD HL,(Vars)		;Check for DFile overflow
	SBC HL,DE		;Test overflow condition
	EX DE,HL		;HL=DFxCC
	JP C LineEnd		;If overflow condition stop printing
	LD (DFxCC),HL		;Set PRINT position
	JR Width		;Loop until C=0
;================================
;Set Reg A to 8bit parameter value
;================================
@SetA
	LD (CHAdd),HL		;Start of parameter
	CALL Scanning		;Call the expression evaluator
	PUSH HL 		;PUSH BASIC line addr
	CALL FPtoA		;A=Results
	POP HL 			;POP BASIC line addr
	RET
;================================
; Get the line length and CODE X$
; parameters
;================================
@GetPars
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;Line length address
	CALL SetA		;Get Length
	AND A			;Length=0
	JR Z ErrorB		;Yes, error B
	LD (Count+1),A		;Set length counter
	INC HL			;CODE X$ address
	CALL SetA		;Get CODE X$
@Count
	LD B,00			;B=length
	RET			;A=CODE X$, B=Length
@ErrorB
	RST 08			;Error restart
	0A			;Error Report B: Integer out of range
;================================
; Check for EOL
;================================
@LineEnd
	POP HL			;POP CHAdd
	LD (CHAdd),HL		;Set CHAdd
	LD A,(HL)		;Test for EOL marker
	CP 76			;EOL?
        JR Z +4			;Yes, return via ROMs "NEXT-LINE" routine
	CALL Print1		;Else start next PRINT statement
	RET
	JP SPRet
;================================
; Delay loop
;================================
@Delay
	LD BC,0001	 	;Delay Loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ Delay+3
	RET
;================================
; Select BKG/Paper
;================================
@Ink
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;HL=X1 parameter
	CALL SetA		;Get color definition
	LD (SetInk+1),A		;Set mode
	LD HL,C000		;Color attribute file
        LD DE,C001		;+1
        LD BC,03FF		;Byte count
@SetInk
        LD (HL),00		;Fill attribute file with color definition
        LDIR
	JP SPRet
;================================
; Pos/Neg color switch
;================================
@Switch
	LD A,(ChroF)		;Test chroma flag
	AND A			;Chroma
	JR NZ AttribSwap	;Yes, move on
	CALL Invert		;Mono invert
	JP Release
@AttribSwap
	LD A,(C008)		;Color attributes
	CP 11			;Positive or Negative?
	JR Z Hues+5		;If + use - table
;================================
; Colors
;================================
@Hues
	LD HL,PosTable		;Positive color definitions
	JR +3
	LD HL,NegTable		;Negative color definitions
        LD DE,C000		;Attribute file destination
        LD BC,0080		;Byte count
        LDIR			;Fill attribute file with color definitions
	JP Release		;Wait for key release then return
;================================
; Pulse
;================================
@Pulse
	CALL Release		;Wait for key release
	LD B,0E			;Counter
@PulseIn
	PUSH BC			;PUSH counter
	CALL RotIn		;Rotate In
	CALL Delay		;Delay loop
	CALL Keyscan		;Check for key press
	POP BC			;POP counter
	CP 01			;Key pressed?
	JP NZ Release		;Wait for key release then return
	DJNZ PulseIn		;Loop until B=0
	LD B,0E			;Counter
@PulseOut
	PUSH BC			;PUSH counter
	CALL RotOut		;Rotate Out
	CALL Delay		;Delay loop
	CALL Keyscan		;Check for key press
	POP BC			;POP counter
	CP 01			;Key pressed?
	JP NZ Release		;Return via Release
	DJNZ PulseOut		;Loop until B=0
	JR Pulse+3
;================================
; Rotate
;================================
@Rotate
	CALL Release		;Wait for key release
	CALL Delay		;Delay loop
	CALL 0000		;Rotate direction
	CALL Keyscan		;Check for key press
	CP 01			;Key pressed?
	JP NZ Release		;Wait for key release then return
	JR Rotate+3		;Loop until keypress
;================================
; Rotate Outwards
;================================
@RotOut
	LD HL,C008		;Attribute file
	LD DE,PRBuff		;Save attribute 1 to PRBuff
	LD BC,0008		;Byte count
	LDIR			;Copy to PRBuff
	LD DE,C008		;Destination=Attribute 0
	LD C,78			;Byte count
	LDIR			;Shift up in attributes file
	LD HL,PRBuff		;Copy attribute 1 to attribute 15
	LD DE,C078		;Destination=Attribute 15
	LD C,08			;Byte count
	LDIR			;Copy from PRBuff to attribute 15
	Ret
;================================
; Rotate Inwards
;================================
@RotIn
	LD HL,C078		;Attribute 15
	LD DE,PRBuff		;Save attribute 15 to PRBuff
	LD BC,0008		;Byte count
	LDIR			;Copy to PRBuff
	LD HL,C077		;Source=Attribute 14 last byte
	LD DE,C07F		;Destination=Attribute 15 last byte
	LD C,78			;Byte count
	LDDR			;Shift down in attributes file
	LD HL,PRBuff		;Copy attribute 15 to attribute 1
	LD DE,C008		;Destination=Attribute 1
	LD C,08			;Byte count
	LDIR			;Copy from PRBuff to attribute 1
	RET
;================================
; Swap B & W
;================================
@Invert				;ZX invert Screen
	LD HL,0001		;Locate altDFile addr
	CALL LAddr		;Find line address
	LD BC,0006		;Skip header
	ADD HL,BC		;Start of altDFile
	LD (altDFile+1),HL	;Set altDFile pointer
	LD HL,(DFile)		;Source=DFile
	PUSH HL			;PUSH DFile pointer
@AltDFile
	LD DE,0000		;DE=altDFile
	LD BC,0319		;Byte count
	LDIR			;Copy DFile to altDFile
	LD BC,0300 		;Byte count
	LD HL,(altDFile+1)	;Invert the altDFile
@InvLoop
	INC HL
	LD A,(HL)		;Load chr. code
	CP 76			;N/L chr?
	JR Z InvLoop		;Yes, skip it by looping back
	ADD A,80		;Add 128 to chr. code
	LD (HL),A		;Display inverted chr. code
	DEC BC 			;Decrease byte counter
	LD A,B 
	OR C 
	JR NZ InvLoop		;Loop until BC=0
	LD HL,(altDFile+1)	;Source=altDFile pointer
	LD (DFile),HL		;Set (DFile) to altDFile
				;Display altDFile while updateing the real DFile
	POP DE			;POP DFile pointer
	PUSH DE			;PUSH DFile pointer
	LD BC,0319		;Byte count
	LDIR			;Copy altDFile to DFile
	POP HL			;POP DFile pointer
	LD (DFile),HL		;Set (DFile) to back to normal
	RET
;================================
; B&W cycle
;================================
@BWCycle
	CALL Release		;Wait for key release
	CALL Delay		;Delay loop
	CALL Invert
	CALL Keyscan		;Check for key press
	CP 01
	JP NZ Release		;Wait for key release then return
	JR BWCycle+3		;No, keep looping
;================================
; USAGE:
;
;  PRINT USR PokeDF;Y,X,CODE C$
;
;   Y=Ycoord
;   X=Xcoord
;   C$=Chr to POKE
;================================
@PokeDF
	LD HL,(CHAdd)		;HL=CHAdd
	INC HL			;Ycoord addr
	CALL SetA		;Get Ycoord
	LD (RowCount+1),A	;Set row counter
	INC HL			;Xcoord addr
	PUSH HL			;PUSH CHadd
	LD HL,(DFile)		;Get DFile+1
	INC HL
@RowCount
	LD A,00			;Add A rows to DFile
	AND A			;Rows=0?
	JR Z Rows0		;Yes, move on
	LD B,A			;B=row counter
	LD DE,0021		;33 bytes/row
@AddRow
	ADD HL,DE
	DJNZ AddRow
@Rows0
	LD (RowBytes+1),HL	;Set DFile Ycoord offset
	POP HL			;POP CHadd
	CALL SetA		;Get Xcoord
	LD (RowBytes+4),A	;Set Xcoord
	INC HL			;POKE value addr
	CALL SetA		;Get POKE value
	PUSH HL			;PUSH CHadd
@RowBytes
	LD HL,0000		;HL=DFile+33*Ycoord
	LD BC,0000		;BC=Xcoord
	ADD HL,BC		;HL=DFile+33*Ycoord+Xcoord
	LD (HL),A		;POKE DFile
	POP HL			;POP CHadd
	LD A,(HL)		;Test for EOL marker
	CP 76			;EOL?
	JP Z SPRet		;Yes, return via ROMs "NEXT-LINE" routine
	JR PokeDF+3		;Next POKE set
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Wait for key release
;================================
@Release
	CALL Keyboard		;Scan keyboard routine.
	INC L			;
	JR NZ Release		;Loop until key released
@Ret0
	LD BC,0000
	RET
;================================
; Sort out the key presses
;================================
@KeySort
	CALL Keyscan
	CP 01
	JR Z KeySort		;Loop until keypress
	CP "N"			;"N" key?
	JR NZ +5		;No, move on
	CALL Switch
	JR KeySort		;Loop until "Q" pressed
	CP "I"			;"I" key?
	JR NZ KeyO		;No, move on
	LD A,(ChroF)		;Test chroma flag
	AND A			;Chroma?
	JR NZ +5		;Yes, move on
	CALL BWCycle		;Cycle B&W
	JR KeySort		;Loop until "Q" pressed
	LD HL,RotIn		;Set for rotate in
	LD A,"I"		;Set switch key to "I"
	JR RotSet
@KeyO
	CP "O"
	JR NZ KeyP		;No, move on
	LD A,(ChroF)		;Test chroma flag
	AND A			;Chroma?
	JR Z KeySort		;No, Loop until "Q" pressed
	LD HL,RotOut		;Set for rotate out
	LD A,"O"		;Set switch key to "O"
@RotSet
	LD(Rotate+7),HL		;Set rotation direction
	CALL Rotate		;Start rotation
	JR KeySort		;Loop until "Q" pressed
@KeyP
	CP "P"
	JR NZ KeyS
	CALL Pulse
@KeyS
	CP "S"
	JR NZ KeyQ		;No, move on
	CALL RotIn
	CALL Release
	JR KeySort
@KeyQ
	CP "Q"
	JR Z Release		;Return to BASIC
	CP 76
	JR Z Release		;Return to BASIC
	JR KeySort		;Loop until "Q" or N/L pressed
;================================
; Palettes
;================================
;Solid Colors
;00 	Black
;11 	D. BLUE
;22 	D. RED 
;33 	D. MAGENTA
;44 	D. GREEN
;55 	D. CYAN
;66 	D. YELLOW
;77 	Grey
;88 	D.Grey
;99 	BLUE
;AA 	RED
;BB 	MAGENTA
;CC 	GREEN
;DD 	CYAN
;EE 	YELLOW
;FF 	WHITE
;================================
;Color attributes
;================================
@PosTable
	00 00 00 00 00 00 00 00		;Black		" "	C000
	11 11 11 11 11 11 11 11 	;D. Blue	g+1	C008
	22 22 22 22 22 22 22 22		;D. Red		g+2	C010
	33 33 33 33 33 33 33 33		;D. Magenta	g+7	C018
	44 44 44 44 44 44 44 44		;D. Green	g+4	C020
	55 55 55 55 55 55 55 55		;D. Cyan	g+5	C028
	66 66 66 66 66 66 66 66		;D. Yellow	g+T	C030
	77 77 77 77 77 77 77 77		;Grey		g+E	C038
	00 00 00 00 00 00 00 00		;Black		g+A	C040
	99 99 99 99 99 99 99 99		;Blue		g+D	C048
	AA AA AA AA AA AA AA AA		;Red		g+S	C050
	BB BB BB BB BB BB BB BB		;Magenta 	"	C058
	CC CC CC CC CC CC CC CC		;Green			C060
	DD DD DD DD DD DD DD DD		;Cyan		$	C068
	EE EE EE EE EE EE EE EE		;Yellow		:	C070
	FF FF FF FF FF FF FF FF		;White   	?	C078
;================================
@NegTable
	00 00 00 00 00 00 00 00		;Black
	EE EE EE EE EE EE EE EE		;D.Blue    -> Yellow
	DD DD DD DD DD DD DD DD		;D.Red     -> Cyan
	CC CC CC CC CC CC CC CC		;D.Magenta -> Green
	BB BB BB BB BB BB BB BB		;D.Green   -> Magenta
	AA AA AA AA AA AA AA AA		;D.Cyan    -> Red
	99 99 99 99 99 99 99 99		;D.Yellow  -> Blue
	00 00 00 00 00 00 00 00		;D.White   -> Black
	FF FF FF FF FF FF FF FF		;Black     -> White
	66 66 66 66 66 66 66 66		;Blue      -> D. Yellow
	55 55 55 55 55 55 55 55		;Red       -> D. Cyan
	44 44 44 44 44 44 44 44		;Magenta   -> D. Green
	33 33 33 33 33 33 33 33		;Green     -> D. Magenta
	22 22 22 22 22 22 22 22		;Cyan      -> D. Red
	11 11 11 11 11 11 11 11 	;Yelloe    -> D. Blue
	00 00 00 00 00 00 00 00		;White     -> Black
;================================